@@ -13,159 +13,17 @@ from urllib2 import urlopen
 import time
 import re
 
-# we make sure only one thread is modifying db messages a time
-outgoing_db_lock = Lock()
-
-# our worker threads
-outgoing_worker_threads = []
-
-# whether we are currently sending mass messages, where we will queue up
-# many messages at once, used as an optimization
-sending_mass_messages = False
-
-def start_sending_mass_messages():
-    global sending_mass_messages
-    sending_mass_messages = True
-
-def stop_sending_mass_messages():
-    global sending_mass_messages
-    sending_mass_messages = False
-
-class HttpRouterThread(Thread, LoggerMixin):
+def fetch_url(url, params):
     """
-    This thread is just a worker thread for messages.  The run() method pops off a message to work on
-    and continues appropriately.
+    Wrapper around url open, mostly here so we can monkey patch over it in unit tests, though
+    in some cases apps may monkey patch this to deal with secondary urls.
     """
-
-    def __init__(self, **kwargs):
-        Thread.__init__(self, **kwargs)
-        self._isbusy = False
-
-    def is_busy(self):
-        return self._isbusy
-
-    def run(self):
-        global sending_mass_messages
-
-        while self.is_alive():
-            if not sending_mass_messages:
-                transaction.enter_transaction_management()
-
-                try:
-                    # this gets any outgoing messages which are either pending or queued
-                    to_process = list(Message.objects.filter(direction='O',
-                                                             status__in=['P','Q']).order_by('status').for_single_update())
-
-                    if len(to_process):
-                        self._isbusy = True
-                        outgoing_message = to_process[0]
-                        outgoing_message.status = 'L'
-                        outgoing_message.save()
-
-                        # frees our update lock
-                        transaction.commit()                        
-
-                        # process the outgoing phases for this message
-                        send_msg = get_router().process_outgoing_phases(outgoing_message)
-
-                        # if it wasn't cancelled, send it off
-                        if send_msg:
-                            self.send_message(outgoing_message)
-                except:
-                    import traceback
-                    traceback.print_exc()
-
-                finally:
-                    try:
-                        transaction.commit()
-                    except:
-                        pass
-
-            self._isbusy = False
-            time.sleep(0.5)
-
-    def fetch_url(self, url):
-        """
-        Wrapper around url open, mostly here so we can monkey patch over it in unit tests.
-        """
+    if getattr(settings, 'ROUTER_HTTP_METHOD', 'GET') == 'GET':
         response = urlopen(url, timeout=15)
-        return response.getcode()
+    else:
+        response = urlopen(url, " ", timeout=15)
 
-    def build_send_url(self, msg, kwargs):
-        """
-        Constructs an appropriate send url for the given message.
-        """
-        # first build up our list of parameters
-        params = {
-            'backend': msg.connection.backend.name,
-            'recipient': msg.connection.identity,
-            'text': msg.text,
-            'id': msg.pk
-        }
-
-        # make sure our parameters are URL encoded
-        params.update(kwargs)
-        for k, v in params.items():
-            try:
-                params[k] = quote_plus(str(v))
-            except UnicodeEncodeError:
-                params[k] = quote_plus(str(v.encode('UTF-8')))
-
-        # get our router URL
-        router_url = settings.ROUTER_URL
-
-        # is this actually a dict?  if so, we want to look up the appropriate backend
-        if type(router_url) is dict:
-            router_dict = router_url
-            backend_name = msg.connection.backend.name
-            
-            # is there an entry for this backend?
-            if backend_name in router_dict:
-                router_url = router_dict[backend_name]
-
-            # if not, look for a default backend 
-            elif 'default' in router_dict:
-                router_url = router_dict['default']
-
-            # none?  blow the hell up
-            else:
-                self.error("No router url mapping found for backend '%s', check your settings.ROUTER_URL setting" % backend_name)
-                raise Exception("No router url mapping found for backend '%s', check your settings.ROUTER_URL setting" % backend_name)
-
-        # return our built up url with all our variables substituted in
-        full_url = router_url % params
-
-        return full_url
-
-    def send_message(self, msg, **kwargs):
-        """
-        Sends the message off.  We first try to directly contact our sms router to deliver it, 
-        if we fail, then we just add it to our outgoing queue.
-        """
-        global outgoing_db_lock
-
-        # and actually hand the message off to our router URL
-        try:
-            url = self.build_send_url(msg, kwargs)
-            status_code = self.fetch_url(url)
-            outgoing_db_lock.acquire()
-            # kannel likes to send 202 responses, really any
-            # 2xx value means things went okay
-            if int(status_code/100) == 2:
-                self.info("SMS[%d] SENT" % msg.id)
-                msg.status = 'S'
-                msg.save()
-            else:
-                self.error("SMS[%d] Message not sent, got status: %s .. queued for later delivery." % (msg.id, status_code))
-                msg.status = 'Q'
-                msg.save()
-            outgoing_db_lock.release()
-        except Exception as e:
-            self.error("SMS[%d] Message not sent: %s .. queued for later delivery." % (msg.id, str(e)))
-            outgoing_db_lock.acquire()
-            msg.status = 'Q'
-            msg.save()
-            outgoing_db_lock.release()
+    return response
 
 class HttpRouter(object, LoggerMixin):
     """
@@ -173,7 +31,6 @@ class HttpRouter(object, LoggerMixin):
     it is expected that the handle_incoming and handle_outcoming calls are made in the HTTP
     thread.
     """
-
     incoming_phases = ("filter", "parse", "handle", "default", "cleanup")
     outgoing_phases = ("outgoing",)
 
@@ -197,50 +54,36 @@ class HttpRouter(object, LoggerMixin):
         Adds this message to the db.  This is both for logging, and we also keep state
         tied to it.
         """
-        global outgoing_db_lock
-
         # lookup / create this backend
         # TODO: is this too flexible?  Perhaps we should do this upon initialization and refuse 
         # any backends not found in our settings.  But I hate dropping messages on the floor.
         backend, created = Backend.objects.get_or_create(name=backend)
-
         contact = HttpRouter.normalize_number(contact)
 
         # create our connection
         connection, created = Connection.objects.get_or_create(backend=backend, identity=contact)
 
-        # finally, create our db message
-        outgoing_db_lock.acquire()
-
         # force to unicode
         text = unicode(text)
-
         message = Message.objects.create(connection=connection,
                                          text=text,
                                          direction=direction,
                                          status=status)
-        outgoing_db_lock.release()
-
         return message
 
-
     def mark_delivered(self, message_id):
         """
         Marks a message as delivered by the backend.
         """
-        global outgoing_db_lock
-
         message = Message.objects.get(pk=message_id)
-        outgoing_db_lock.acquire()
         message.status = 'D'
+        message.delivered = datetime.datetime.now()
         message.save()
-        outgoing_db_lock.release()
 
     def handle_incoming(self, backend, sender, text):
         """
         Handles an incoming message.
         """
-        global outgoing_db_lock
         # create our db message for logging
         db_message = self.add_message(backend, sender, text, 'I', 'R')
 
@@ -301,10 +144,8 @@ class HttpRouter(object, LoggerMixin):
         except StopIteration:
             pass
 
-        outgoing_db_lock.acquire()
         db_message.status = 'H'
         db_message.save()
-        outgoing_db_lock.release()
         
         db_responses = []
 
@@ -324,43 +165,25 @@ class HttpRouter(object, LoggerMixin):
         """
         Adds a message to our outgoing queue, this is a non-blocking action
         """
-        global outgoing_db_lock
-        outgoing_db_lock.acquire()
         db_message = Message.objects.create(connection=connection,
                                             text=text,
                                             direction='O',
                                             status=status,
                                             in_response_to=source)
-        outgoing_db_lock.release()
-
         self.info("SMS[%d] OUT (%s) : %s" % (db_message.id, str(connection), text))
 
-        global outgoing_worker_threads
+        # process our outgoing phases
+        self.process_outgoing_phases(db_message)
 
-        # if we have no ROUTER_URL configured, then immediately process our outgoing phases
-        # and leave the message in the queue
-        if not getattr(settings, 'ROUTER_URL', None):
-            if self.process_outgoing_phases(db_message):
-                outgoing_db_lock.acquire()
-                db_message.status = 'Q'
-                db_message.save()
-                outgoing_db_lock.release()
+        # if it wasn't cancelled
+        if db_message.status != 'C':
+            # queue it
+            db_message.status = 'Q'
+            db_message.save()
 
-        # otherwise, fire up any threads we need to send the message out
-        else:
-            # check for available worker threads in the pool, add one if necessary
-            num_workers = getattr(settings, 'ROUTER_WORKERS', 5)
-            all_busy = True
-            for worker in outgoing_worker_threads:
-                if not worker.is_busy():
-                    all_busy = False
-                    break
-
-            if all_busy and len(outgoing_worker_threads) < num_workers:
-                worker = HttpRouterThread()
-                worker.daemon = True # they don't need to quit gracefully
-                worker.start()
-                outgoing_worker_threads.append(worker)
+        # if we have a router URL, send the message off
+        if getattr(settings, 'ROUTER_URL', None):
+            db_message.send()
 
         return db_message
                 
@@ -371,7 +194,6 @@ class HttpRouter(object, LoggerMixin):
         """
         # add it to our outgoing queue
         db_message = self.add_outgoing(msg.connection, msg.text, source, status='P')
-        
         return db_message
 
     def process_outgoing_phases(self, outgoing):
@@ -410,10 +232,7 @@ class HttpRouter(object, LoggerMixin):
                 # abort ALL further processing of this message
                 if not send_msg:
                     outgoing.status = 'C'
-
-                    outgoing_db_lock.acquire()
                     outgoing.save()
-                    outgoing_db_lock.release()
 
                     self.warning("Message cancelled")
                     send_msg = False
